import os
import pytest

import librosa
import numpy as np

from audiomentations import SpecChannelShuffle
from audiomentations.core.audio_loading_utils import load_sound_file
from audiomentations.core.transforms_interface import MonoAudioNotSupportedException
from demo.demo import DEMO_DIR
from .utils import plot_matrix

DEBUG = False


def test_shuffle_channels():
    samples, sample_rate = load_sound_file(
        os.path.join(DEMO_DIR, "background_noises", "hens.ogg"),
        sample_rate=None,
        mono=False,
    )
    assert samples.shape[0] == 2
    magnitude_spectrogram_chn0 = librosa.feature.melspectrogram(
        y=np.asfortranarray(samples[0, :]), sr=sample_rate
    )
    magnitude_spectrogram_chn1 = librosa.feature.melspectrogram(
        y=np.asfortranarray(samples[1, :]), sr=sample_rate
    )
    multichannel_magnitude_spectrogram = np.zeros(
        shape=(
            magnitude_spectrogram_chn0.shape[0],
            magnitude_spectrogram_chn0.shape[1],
            3,
        ),
        dtype=np.float32,
    )
    multichannel_magnitude_spectrogram[:, :, 0] = magnitude_spectrogram_chn0
    multichannel_magnitude_spectrogram[:, :, 1] = magnitude_spectrogram_chn1
    multichannel_magnitude_spectrogram[:, :, 2] = magnitude_spectrogram_chn1 * 0.7

    if DEBUG:
        image = (7 + np.log10(multichannel_magnitude_spectrogram + 0.0000001)) / 8
        plot_matrix(image, title="before")

    # Make sure the shuffled channels do not equal the original order
    transform = SpecChannelShuffle(p=1.0)
    for _ in range(100000):
        transform.randomize_parameters(multichannel_magnitude_spectrogram)
        if transform.parameters["shuffled_channel_indexes"] != [0, 1, 2]:
            break
    transform.freeze_parameters()

    augmented_spectrogram = transform(multichannel_magnitude_spectrogram)

    if DEBUG:
        image = (7 + np.log10(augmented_spectrogram + 0.0000001)) / 8
        plot_matrix(image, title="after")

    with np.testing.assert_raises(AssertionError):
        np.testing.assert_array_equal(
            augmented_spectrogram, multichannel_magnitude_spectrogram
        )

    for augmented_index, original_index in enumerate(
        transform.parameters.get("shuffled_channel_indexes")
    ):
        np.testing.assert_array_equal(
            augmented_spectrogram[:, :, augmented_index],
            multichannel_magnitude_spectrogram[:, :, original_index],
        )


def test_shuffle_channels_mono():
    samples, sample_rate = load_sound_file(
        os.path.join(DEMO_DIR, "acoustic_guitar_0.wav"), sample_rate=None
    )
    magnitude_spectrogram = librosa.feature.melspectrogram(y=samples, sr=sample_rate)

    transform = SpecChannelShuffle(p=1.0)
    with pytest.raises(MonoAudioNotSupportedException):
        transform(magnitude_spectrogram)


def test_empty_spectrogram():
    spec = np.zeros(shape=(0, 0), dtype=np.float32)
    transform = SpecChannelShuffle(p=1.0)
    augmented_spectrogram = transform(spec)

    np.testing.assert_array_equal(spec, augmented_spectrogram)



import os

import librosa
import numpy as np

from audiomentations import SpecFrequencyMask
from audiomentations.core.audio_loading_utils import load_sound_file
from demo.demo import DEMO_DIR
from .utils import plot_matrix

DEBUG = False


def test_fill_zeros():
    samples, sample_rate = load_sound_file(
        os.path.join(DEMO_DIR, "acoustic_guitar_0.wav"), sample_rate=None
    )
    magnitude_spectrogram = librosa.feature.melspectrogram(y=samples, sr=sample_rate)

    mask_fraction = 0.05
    transform = SpecFrequencyMask(
        fill_mode="constant",
        fill_constant=0.0,
        min_mask_fraction=mask_fraction,
        max_mask_fraction=mask_fraction,
        p=1.0,
    )
    augmented_spectrogram = transform(magnitude_spectrogram)

    if DEBUG:
        plot_matrix(np.log(augmented_spectrogram))

    with np.testing.assert_raises(AssertionError):
        np.testing.assert_array_equal(augmented_spectrogram, magnitude_spectrogram)

    num_zeroed_frequencies = 0
    for i in range(augmented_spectrogram.shape[0]):
        if sum(augmented_spectrogram[i]) == 0.0:
            num_zeroed_frequencies += 1

    assert num_zeroed_frequencies == int(
        round(magnitude_spectrogram.shape[0] * mask_fraction)
    )


def test_fill_zeros_multichannel():
    samples, sample_rate = load_sound_file(
        os.path.join(DEMO_DIR, "background_noises", "hens.ogg"),
        sample_rate=None,
        mono=False,
    )
    assert samples.shape[0] == 2
    magnitude_spectrogram_chn0 = librosa.feature.melspectrogram(
        y=np.asfortranarray(samples[0, :]), sr=sample_rate
    )
    magnitude_spectrogram_chn1 = librosa.feature.melspectrogram(
        y=np.asfortranarray(samples[1, :]), sr=sample_rate
    )
    multichannel_magnitude_spectrogram = np.zeros(
        shape=(
            magnitude_spectrogram_chn0.shape[0],
            magnitude_spectrogram_chn0.shape[1],
            3,
        ),
        dtype=np.float32,
    )
    multichannel_magnitude_spectrogram[:, :, 0] = magnitude_spectrogram_chn0
    multichannel_magnitude_spectrogram[:, :, 1] = magnitude_spectrogram_chn1
    multichannel_magnitude_spectrogram[:, :, 2] = magnitude_spectrogram_chn1

    mask_fraction = 0.05
    transform = SpecFrequencyMask(
        fill_mode="constant",
        fill_constant=0.0,
        min_mask_fraction=mask_fraction,
        max_mask_fraction=mask_fraction,
        p=1.0,
    )
    augmented_spectrogram = transform(multichannel_magnitude_spectrogram)

    if DEBUG:
        image = (7 + np.log10(augmented_spectrogram + 0.0000001)) / 8
        plot_matrix(image)

    with np.testing.assert_raises(AssertionError):
        np.testing.assert_array_equal(
            augmented_spectrogram, multichannel_magnitude_spectrogram
        )

    num_zeroed_frequencies = 0
    for i in range(augmented_spectrogram.shape[0]):
        if np.sum(augmented_spectrogram[i]) == 0.0:
            num_zeroed_frequencies += 1

    assert num_zeroed_frequencies == int(
        round(multichannel_magnitude_spectrogram.shape[0] * mask_fraction)
    )


def test_fill_mean():
    samples, sample_rate = load_sound_file(
        os.path.join(DEMO_DIR, "acoustic_guitar_0.wav"), sample_rate=None
    )
    magnitude_spectrogram = librosa.feature.melspectrogram(y=samples, sr=sample_rate)

    min_mask_fraction = 0.05
    max_mask_fraction = 0.09
    transform = SpecFrequencyMask(
        fill_mode="mean",
        min_mask_fraction=min_mask_fraction,
        max_mask_fraction=max_mask_fraction,
        p=1.0,
    )
    augmented_spectrogram = transform(magnitude_spectrogram)

    if DEBUG:
        plot_matrix(np.log(augmented_spectrogram))

    num_masked_frequencies = 0
    for i in range(augmented_spectrogram.shape[0]):
        frequency_slice = augmented_spectrogram[i]
        if (
            np.amin(frequency_slice) == np.amax(frequency_slice)
            and sum(frequency_slice) != 0.0
        ):
            num_masked_frequencies += 1

    assert num_masked_frequencies >= int(
        round(magnitude_spectrogram.shape[0] * min_mask_fraction)
    )
    assert num_masked_frequencies <= int(
        round(magnitude_spectrogram.shape[0] * max_mask_fraction)
    )


def test_fill_mean_multichannel():
    samples, sample_rate = load_sound_file(
        os.path.join(DEMO_DIR, "background_noises", "hens.ogg"),
        sample_rate=None,
        mono=False,
    )
    assert samples.shape[0] == 2
    magnitude_spectrogram_chn0 = librosa.feature.melspectrogram(
        y=np.asfortranarray(samples[0, :]), sr=sample_rate
    )
    magnitude_spectrogram_chn1 = librosa.feature.melspectrogram(
        y=np.asfortranarray(samples[1, :]), sr=sample_rate
    )
    multichannel_magnitude_spectrogram = np.zeros(
        shape=(
            magnitude_spectrogram_chn0.shape[0],
            magnitude_spectrogram_chn0.shape[1],
            3,
        ),
        dtype=np.float32,
    )
    multichannel_magnitude_spectrogram[:, :, 0] = magnitude_spectrogram_chn0
    multichannel_magnitude_spectrogram[:, :, 1] = magnitude_spectrogram_chn1
    multichannel_magnitude_spectrogram[:, :, 2] = magnitude_spectrogram_chn1

    mask_fraction = 0.05
    transform = SpecFrequencyMask(
        fill_mode="mean",
        min_mask_fraction=mask_fraction,
        max_mask_fraction=mask_fraction,
        p=1.0,
    )
    augmented_spectrogram = transform(multichannel_magnitude_spectrogram)

    if DEBUG:
        image = (7 + np.log10(augmented_spectrogram + 0.0000001)) / 8
        plot_matrix(image)

    with np.testing.assert_raises(AssertionError):
        np.testing.assert_array_equal(
            augmented_spectrogram, multichannel_magnitude_spectrogram
        )

    num_masked_frequencies = 0
    for i in range(augmented_spectrogram.shape[0]):
        frequency_slice = augmented_spectrogram[i]
        if (
            np.amin(frequency_slice) == np.amax(frequency_slice)
            and np.sum(frequency_slice) != 0.0
        ):
            num_masked_frequencies += 1

    assert num_masked_frequencies == int(
        round(multichannel_magnitude_spectrogram.shape[0] * mask_fraction)
    )



import numpy as np
from numpy.testing import assert_array_equal

from audiomentations import SpecChannelShuffle, SpecFrequencyMask, SpecCompose


def test_freeze_and_unfreeze_parameters():
    spectrogram = np.random.random((256, 256, 3))
    augmenter = SpecCompose(
        [
            SpecChannelShuffle(p=1.0),
            SpecFrequencyMask(p=1.0),
        ]
    )
    perturbed_samples1 = augmenter(magnitude_spectrogram=spectrogram)
    augmenter.freeze_parameters()
    for transform in augmenter.transforms:
        assert transform.are_parameters_frozen == True
    perturbed_samples2 = augmenter(magnitude_spectrogram=spectrogram)

    assert_array_equal(perturbed_samples1, perturbed_samples2)

    augmenter.unfreeze_parameters()
    for transform in augmenter.transforms:
        assert transform.are_parameters_frozen == False

def test_randomize_parameters_and_apply():
    spectrogram = np.random.random((256, 256, 3))
    augmenter = SpecCompose(
        [
            SpecChannelShuffle(p=1.0),
            SpecFrequencyMask(p=1.0),
        ]
    )
    augmenter.freeze_parameters()
    augmenter.randomize_parameters(magnitude_spectrogram=spectrogram)

    parameters = [transform.parameters for transform in augmenter.transforms]

    perturbed_samples1 = augmenter(magnitude_spectrogram=spectrogram)
    perturbed_samples2 = augmenter(magnitude_spectrogram=spectrogram)

    assert_array_equal(perturbed_samples1, perturbed_samples2)

    augmenter.unfreeze_parameters()

    for transform_parameters, transform in zip(parameters, augmenter.transforms):
        assert transform_parameters == transform.parameters
        assert transform.are_parameters_frozen == False



import numpy as np
import pytest

from audiomentations import TimeMask, Compose


def test_apply_time_mask():
    sample_len = 1024
    samples_in = np.random.normal(0, 1, size=sample_len).astype(np.float32)
    sample_rate = 16000
    augmenter = TimeMask(min_band_part=0.2, max_band_part=0.5, p=1.0)

    samples_out = augmenter(samples=samples_in, sample_rate=sample_rate)
    assert samples_out.dtype == np.float32
    assert len(samples_out) == sample_len

    std_in = np.mean(np.abs(samples_in))
    std_out = np.mean(np.abs(samples_out))
    assert std_out < std_in


def test_invalid_params():
    with pytest.raises(ValueError):
        TimeMask(min_band_part=0.5, max_band_part=1.5)

    with pytest.raises(ValueError):
        TimeMask(min_band_part=-0.5, max_band_part=0.5)

    with pytest.raises(ValueError):
        TimeMask(min_band_part=0.6, max_band_part=0.5)


def test_apply_time_mask_multichannel():
    sample_len = 1024
    samples_in = np.random.normal(0, 1, size=(2, sample_len)).astype(np.float32)
    sample_rate = 16000
    augmenter = TimeMask(min_band_part=0.2, max_band_part=0.5, p=1.0)

    samples_out = augmenter(samples=samples_in, sample_rate=sample_rate)
    assert samples_out.dtype == np.float32
    assert samples_out.shape == samples_in.shape

    std_in = np.mean(np.abs(samples_in))
    std_out = np.mean(np.abs(samples_out))
    assert std_out < std_in


def test_apply_time_mask_with_fade():
    sample_len = 1024
    samples_in = np.random.normal(0, 1, size=sample_len).astype(np.float32)
    sample_rate = 16000
    augmenter = TimeMask(min_band_part=0.2, max_band_part=0.5, fade=True, p=1.0)

    samples_out = augmenter(samples=samples_in, sample_rate=sample_rate)
    assert samples_out.dtype == np.float32
    assert len(samples_out) == sample_len

    std_in = np.mean(np.abs(samples_in))
    std_out = np.mean(np.abs(samples_out))
    assert std_out < std_in


def test_apply_time_mask_with_fade_short_signal():
    sample_len = 100
    samples_in = np.random.normal(0, 1, size=sample_len).astype(np.float32)
    sample_rate = 16000
    augmenter = TimeMask(min_band_part=0.2, max_band_part=0.5, fade=True, p=1.0)

    samples_out = augmenter(samples=samples_in, sample_rate=sample_rate)
    assert samples_out.dtype == np.float32
    assert len(samples_out) == sample_len

    std_in = np.mean(np.abs(samples_in))
    std_out = np.mean(np.abs(samples_out))
    assert std_out < std_in
